# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import json

from pyvcloud.vcd.client import ApiVersion
import yaml

import container_service_extension.client.constants as cli_constants
from container_service_extension.client.tkgclient import TkgClusterApi
from container_service_extension.client.tkgclient.api_client import ApiClient
from container_service_extension.client.tkgclient.configuration import Configuration  # noqa: E501
from container_service_extension.client.tkgclient.models.tkg_cluster import TkgCluster  # noqa: E501
import container_service_extension.client.tkgclient.rest as tkg_s_rest
import container_service_extension.client.utils as client_utils
import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
import container_service_extension.common.utils.core_utils as utils
import container_service_extension.common.utils.pyvcloud_utils as vcd_utils
import container_service_extension.exception.exceptions as cse_exceptions
import container_service_extension.logging.logger as logger
import container_service_extension.rde.acl_service as cluster_acl_svc
from container_service_extension.rde.constants import CLUSTER_ACL_LIST_FIELDS, Nss, Vendor  # noqa: E501
import container_service_extension.rde.models.common_models as common_models


class DEClusterTKGS:
    """Embedded Kubernetes into vSphere."""

    # NOTE: When converting model objects from tkgclient/models to dictionary,
    # please use utils.swagger_object_to_dict() function. This preserves camel
    # case of the keys.

    def __init__(self, client):
        self._client = client
        tkg_s_config = Configuration()
        tkg_s_config.host = f"{self._client.get_cloudapi_uri()}/1.0.0/"
        tkg_s_config.verify_ssl = self._client._verify_ssl_certs

        self._tkg_s_client = ApiClient(configuration=tkg_s_config)
        jwt_token = self._client.get_access_token()
        if jwt_token:
            self._tkg_s_client.set_default_header(
                cli_constants.TKGRequestHeaderKey.AUTHORIZATION,
                f"Bearer {jwt_token}"
            )
        else:
            legacy_token = self._client.get_xvcloud_authorization_token()
            self._tkg_s_client.set_default_header(cli_constants.TKGRequestHeaderKey.X_VCLOUD_AUTHORIZATION,  # noqa: E501
                                                  legacy_token)
        # get highest supported api version
        api_version = self._client.get_api_version()
        supported_api_versions = self._client.get_supported_versions_list()
        max_api_version = utils.get_max_api_version(supported_api_versions)

        # api_version = self._client.get_api_version()
        # use api_version 37.0.0-alpha if VCD 10.3 is used.
        if float(max_api_version) >= float(ApiVersion.VERSION_36.value):
            api_version = '37.0.0-alpha'
        self._tkg_s_client.set_default_header(cli_constants.TKGRequestHeaderKey.ACCEPT,  # noqa: E501
                                              f"application/json;version={api_version}")  # noqa: E501
        self._tkg_s_client_api = TkgClusterApi(api_client=self._tkg_s_client)

    def set_tenant_org_context(self, org_name=None):
        """Set tenant org context if not set in the client.

        :param str org_name: Name of the org. If not set, makes use of
            the logged in org name
        """
        if cli_constants.TKGRequestHeaderKey.X_VMWARE_VCLOUD_TENANT_CONTEXT not in self._tkg_s_client.default_headers:  # noqa: E501
            logger.CLIENT_LOGGER.debug(f"Setting client org context with org name {org_name}")  # noqa: E501
            org_resource = vcd_utils.get_org(self._client, org_name=org_name)
            org_id = org_resource.href.split('/')[-1]
            self._tkg_s_client.set_default_header(cli_constants.TKGRequestHeaderKey.X_VMWARE_VCLOUD_TENANT_CONTEXT,  # noqa: E501
                                                  org_id)

    def get_tkg_s_cluster(self, cluster_id):
        """Sample method to use tkg_s_client.

        To be modified or removed as per the needs of CSE-CLI

        :param cluster_id: Id of the cluster
        :return: Tkg cluster
        :rtype: dict
        """
        # Returns tuple of response_data, response_status, response_headers,
        #   tkg_def_entities
        try:
            response = self._tkg_s_client_api.get_tkg_cluster(
                cluster_id,
                _return_http_data_only=False)
            cluster: TkgCluster = response[0]
            cluster_def_entity = response[3]
            return cluster, cluster_def_entity
        except Exception as e:
            logger.CLIENT_LOGGER.debug(e)
            raise

    def list_tkg_s_clusters(self, vdc=None, org=None):
        """List all the TKG-S clusters.

        :param str vdc: name of vdc to filter by
        :param str org: name of the org to filter by
        :return: list of TKG-S cluster information.
        :rtype: List[dict]
        """
        self.set_tenant_org_context(org_name=org)
        filters = {}
        if vdc:
            filters[cli_constants.TKGEntityFilterKey.VDC_NAME.value] = vdc
        filter_string = utils.construct_filter_string(filters)
        query_params = {
            shared_constants.PaginationKey.PAGE_NUMBER.value: shared_constants.CSE_PAGINATION_FIRST_PAGE_NUMBER,  # noqa: E501
            shared_constants.PaginationKey.PAGE_SIZE.value: shared_constants.CSE_PAGINATION_DEFAULT_PAGE_SIZE  # noqa: E501
        }
        has_more_results = True
        while has_more_results:
            (entities, status, headers, additional_details) = \
                self._tkg_s_client_api.list_tkg_clusters(
                    f"{Vendor.VMWARE}/{Nss.TKG}/1.0.0",
                    _return_http_data_only=False,
                    object_filter=filter_string,
                    query_params=query_params)
            # tkg_def_entities in the following statement represents the
            # information associated with the defined entity
            tkg_def_entities = additional_details['entityDetails']
            clusters = []
            for i in range(len(entities)):
                # NOTE: tkg_def_entities will contain corresponding defined
                # entity details
                entity: TkgCluster = entities[i]
                entity_properties = tkg_def_entities[i]
                logger.CLIENT_LOGGER.debug(f"TKG-S Defined entity list from server: {entity}")  # noqa: E501
                cluster = {
                    cli_constants.CLIOutputKey.CLUSTER_NAME.value: entity.metadata.name,  # noqa: E501
                    cli_constants.CLIOutputKey.VDC.value: entity.metadata.virtual_data_center_name,  # noqa: E501
                    cli_constants.CLIOutputKey.ORG.value: entity_properties['org']['name'],  # noqa: E501
                    cli_constants.CLIOutputKey.K8S_RUNTIME.value: shared_constants.ClusterEntityKind.TKG_S.value,  # noqa: E501
                    cli_constants.CLIOutputKey.K8S_VERSION.value: entity.spec.distribution.version,  # noqa: E501
                    cli_constants.CLIOutputKey.STATUS.value: entity.status.phase if entity.status else 'N/A',   # noqa: E501
                    cli_constants.CLIOutputKey.OWNER.value: entity_properties['owner']['name'],  # noqa: E501
                }
                clusters.append(cluster)
            has_more_results = additional_details['page'] < additional_details['pageCount']  # noqa: E501
            yield clusters, has_more_results
            query_params[shared_constants.PaginationKey.PAGE_NUMBER.value] += 1

    def get_tkg_s_clusters_by_name(self, name, vdc=None, org=None):
        self.set_tenant_org_context(org_name=org)
        filters = {cli_constants.TKGEntityFilterKey.CLUSTER_NAME.value: name}
        if vdc:
            filters[cli_constants.TKGEntityFilterKey.VDC_NAME.value] = vdc
        filter_string = utils.construct_filter_string(filters)
        response = \
            self._tkg_s_client_api.list_tkg_clusters(
                f"{Vendor.VMWARE}/{Nss.TKG}/1.0.0",
                object_filter=filter_string)
        tkg_entities = []
        tkg_def_entities = []
        if response:
            tkg_entities = response[0]
            tkg_def_entities = response[3]['entityDetails']
        if len(tkg_entities) == 0:
            raise cse_exceptions.ClusterNotFoundError(
                f"TKG-S cluster with name '{name}' not found.")
        if len(tkg_entities) > 1:
            if not org:
                raise cse_exceptions.CseDuplicateClusterError(
                    f"Multiple clusters with the name '{name}' found."
                    " Please specify the org using --org flag.")
            raise cse_exceptions.CseDuplicateClusterError(
                f"Multiple clusters with the name '{name}' present in the"
                "same Organization. Please contact the administrator.")
        return tkg_entities, tkg_def_entities

    def get_cluster_info(self, cluster_name, cluster_id=None,
                         org=None, vdc=None, **kwargs):
        """Get cluster information of a TKG-S cluster API.

        :param str cluster_name: name of the cluster
        :param str cluster_id: id of the cluster (optional)
        :param str vdc: name of vdc
        :param str org: name of org

        :return: cluster information
        :rtype: str
        :raises ClusterNotFoundError
        """
        if cluster_id:
            return self.get_cluster_info_by_id(cluster_id, org=org)
        try:
            tkg_entities, _ = \
                self.get_tkg_s_clusters_by_name(cluster_name, vdc=vdc, org=org)
            cluster_entity_dict = client_utils.swagger_object_to_dict(tkg_entities[0])  # noqa: E501
            logger.CLIENT_LOGGER.debug(
                f"Received defined entity of cluster {cluster_name} : {cluster_entity_dict}")  # noqa: E501
            return yaml.dump(cluster_entity_dict)
        except tkg_s_rest.ApiException as e:
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{e.reason}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"Error deleting cluster: {e}")
            raise

    def get_cluster_info_by_id(self, cluster_id, org=None, **kwargs):
        """Get TKG-S cluster information using cluster ID.

        :param str cluster_id:
        :param str org:
        :return: cluster information in yaml format
        :rtype: string
        """
        try:
            self.set_tenant_org_context(org_name=org)
            tkg_entity, _ = self.get_tkg_s_cluster(cluster_id)
            cluster_info = client_utils.swagger_object_to_dict(tkg_entity)
            logger.CLIENT_LOGGER.debug(f"Retrieved TKG-S entity for ID {cluster_id}: {cluster_info}")  # noqa: E501
            return yaml.dump(cluster_info)
        except tkg_s_rest.ApiException as e:
            logger.CLIENT_LOGGER.debug(e)
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{e.reason}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"Error getting TKG-S cluster information: {e}")  # noqa: E501
            raise

    def apply(self, cluster_config: dict, cluster_id=None, org=None, **kwargs):
        """Apply the configuration either to create or update the cluster.

        :param dict cluster_config: cluster configuration information
        :param str cluster_id:
        :param str org:

        :return: string containing the task href for the operation
        :rtype: str
        """
        try:
            self.set_tenant_org_context(org_name=org)
            cluster_name = cluster_config.get('metadata', {}).get('name')
            vdc_name = cluster_config.get('metadata', {}).get('virtualDataCenterName')  # noqa: E501
            try:
                if cluster_id:
                    tkg_entity, tkg_def_entity = self.get_tkg_s_cluster(cluster_id)  # noqa: E501
                else:
                    tkg_entities, tkg_def_entities = \
                        self.get_tkg_s_clusters_by_name(cluster_name, vdc=vdc_name)  # noqa: E501
                    tkg_entity = tkg_entities[0]
                    tkg_def_entity = tkg_def_entities[0]
                cluster_id = tkg_def_entity.get('id')
                cluster_config['metadata']['resourceVersion'] = tkg_entity.metadata.resource_version  # noqa: E501
                response = \
                    self._tkg_s_client_api.update_tkg_cluster_with_http_info(
                        tkg_cluster_id=cluster_id,
                        tkg_cluster=cluster_config)
            except cse_exceptions.ClusterNotFoundError:
                response = \
                    self._tkg_s_client_api.create_tkg_cluster_with_http_info(tkg_cluster=cluster_config)  # noqa: E501
            # Get the task href from the header
            headers = response[2]
            return client_utils.construct_task_console_message(headers.get('Location'))  # noqa: E501
        except tkg_s_rest.ApiException as e:
            server_message = json.loads(e.body).get('message') or e.reason
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{server_message}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"Error while applying cluster spec: {e}")  # noqa: E501
            raise

    def delete_cluster_by_id(self, cluster_id, org=None, **kwargs):
        """Delete a cluster using the cluster id.

        :param str cluster_id:
        :param str org:

        :return: string containing the task href of delete cluster operation
        """
        try:
            self.set_tenant_org_context(org_name=org)
            response = \
                self._tkg_s_client_api.delete_tkg_cluster_with_http_info(tkg_cluster_id=cluster_id)  # noqa: E501
            headers = response[2]
            return client_utils.construct_task_console_message(headers.get('Location'))  # noqa: E501
        except tkg_s_rest.ApiException as e:
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{e.reason}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"Error deleting cluster: {e}")
            raise

    def delete_cluster(self, cluster_name, cluster_id=None,
                       org=None, vdc=None):
        """Delete TKG-S cluster by name.

        :param str cluster_name: TKG-S cluster name
        :param str cluster_id: id of the cluster (optional)
        :param str org: name of the org
        :param str vdc: name of the vdc
        :return: string containing delete cluster task href
        :rtype: str
        :raises ClusterNotFoundError, CseDuplicateClusterError
        """
        try:
            if not cluster_id:
                _, tkg_def_entities = \
                    self.get_tkg_s_clusters_by_name(cluster_name, org=org, vdc=vdc)  # noqa: E501
                cluster_id = tkg_def_entities[0]['id']
            return self.delete_cluster_by_id(cluster_id)
        except tkg_s_rest.ApiException as e:
            server_message = json.loads(e.body).get('message') or e.reason
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{server_message}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"{e}")
            raise

    def get_cluster_config_by_id(self, cluster_id, org=None, **kwargs):
        """Get TKG-S cluster config by cluster id.

        :param str cluster_id: ID of the cluster
        :param str org: URN of the org
        :return the cluster config of the TKG-S cluster
        :rtype: str
        """
        try:
            self.set_tenant_org_context(org_name=org)
            response = self._tkg_s_client_api.create_tkg_cluster_config_task(id=cluster_id)  # noqa: E501
            # Extract the task for creating the config from the Location header
            headers = response[2]
            config_task_href = headers.get('Location')
            if not config_task_href:
                raise Exception(f"Failed to fetch kube-config for TKG-S cluster {cluster_id}")  # noqa: E501
            config_task = self._client.get_resource(config_task_href)
            self._client.get_task_monitor().wait_for_success(config_task)
            config_task = self._client.get_resource(config_task_href)
            return {shared_constants.RESPONSE_MESSAGE_KEY: config_task.Result.ResultContent.text}  # noqa: E501
        except tkg_s_rest.ApiException as e:
            server_message = json.loads(e.body).get('message') or e.reason
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{server_message}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"{e}")
            raise

    def get_cluster_config(self, cluster_name, cluster_id=None,
                           org=None, vdc=None):
        """Get TKG-S cluster config by cluster name.

        :param str cluster_name: name of the cluster
        :param str cluster_id: ID of the cluster
        :param str org: name of the org
        :param str vdc: name of the vdc
        """
        try:
            if not cluster_id:
                _, tkg_def_entities = \
                    self.get_tkg_s_clusters_by_name(cluster_name, org=org, vdc=vdc)  # noqa: E501
                cluster_id = tkg_def_entities[0]['id']
            return self.get_cluster_config_by_id(cluster_id,
                                                 org=org)
        except tkg_s_rest.ApiException as e:
            server_message = json.loads(e.body).get('message') or e.reason
            msg = cli_constants.TKG_RESPONSE_MESSAGES_BY_STATUS_CODE.get(e.status, f"{server_message}")  # noqa: E501
            logger.CLIENT_LOGGER.error(msg)
            raise Exception(msg)
        except Exception as e:
            logger.CLIENT_LOGGER.error(f"{e}")
            raise

    def get_upgrade_plan(self, cluster_name, vdc=None, org=None):
        """List of clusters the TKG-S cluster can upgrade to.

        :param str cluster_name: name of the cluster
        :param str org: name of the org
        :param str vdc: name of the vdc
        """
        raise NotImplementedError(
            "Get Cluster upgrade-plan not supported for TKG-S clusters")

    def upgrade_cluster(self, cluster_name, template_name,
                        template_revision, **kwargs):
        """Upgrade TKG-S cluster to the given distribution."""
        raise NotImplementedError(
            "Cluster upgrade not supported for TKG-S clusters")

    def get_cluster_id_by_name(self, cluster_name, org=None, vdc=None):
        _, tkg_def_entities = \
            self.get_tkg_s_clusters_by_name(cluster_name, org=org, vdc=vdc)
        return tkg_def_entities[0]['id']

    def share_cluster(self, cluster_id, cluster_name, users: list,
                      access_level_id, org=None, vdc=None):
        """Share the cluster with the users in user_name_to_id_dict.

        :param str cluster_id: cluster id
        :param str cluster_name: cluster name
        :param list users: users to share cluster with
        :param str access_level_id: access level id of shared users
        :param str vdc: name of the vdc where the cluster is
        :param str org: name of the org where the users are
        """
        if not cluster_id:
            cluster_id = self.get_cluster_id_by_name(cluster_name, org, vdc)

        # Ensure current cluster user access level is not reduced
        org_href = self._client.get_org_by_name(org).get('href')
        name_to_id: dict = client_utils.create_user_name_to_id_dict(
            self._client, set(users), org_href)
        org_user_id_to_name_dict = vcd_utils.create_org_user_id_to_name_dict(
            self._client, org)
        acl_svc = cluster_acl_svc.ClusterACLService(cluster_id, self._client)
        for acl_entry in acl_svc.list_def_entity_acl_entries():
            username = org_user_id_to_name_dict.get(acl_entry.memberId)
            if name_to_id.get(username):
                curr_access_level = acl_entry.accessLevelId  # noqa: E501
                if client_utils.access_level_reduced(access_level_id,
                                                     curr_access_level):
                    raise Exception(f'{username} currently has higher access '
                                    f'level: {curr_access_level}')

        # share TKG-S def entity
        acl_entry = common_models.ClusterAclEntry(
            grantType=shared_constants.MEMBERSHIP_GRANT_TYPE,
            accessLevelId=access_level_id,
            memberId=None)
        for _, user_id in name_to_id.items():
            acl_entry.memberId = user_id
            acl_svc.share_def_entity(acl_entry)

    def unshare_cluster(self, cluster_id, cluster_name, users: list, org=None,
                        vdc=None):
        if not cluster_id:
            cluster_id = self.get_cluster_id_by_name(cluster_name, org, vdc)

        # Get acl entry ids for users
        org_href = self._client.get_org_by_name(org).get('href')
        name_to_id: dict = client_utils.create_user_name_to_id_dict(
            self._client, set(users), org_href)
        users_ids: set = {user_id for _, user_id in name_to_id.items()}
        acl_svc = cluster_acl_svc.ClusterACLService(cluster_id, self._client)
        delete_acl_ids = []
        for acl_entry in acl_svc.list_def_entity_acl_entries():
            if acl_entry.memberId in users_ids:
                delete_acl_ids.append(acl_entry.id)
                users_ids.remove(acl_entry.memberId)

        if len(users_ids) > 0:
            org_user_id_to_name_dict = \
                vcd_utils.create_org_user_id_to_name_dict(self._client, org)
            missing_users = [org_user_id_to_name_dict[user_id] for user_id in users_ids]  # noqa: E501
            raise Exception(f'Cluster {cluster_name or cluster_id} is not '
                            f'currently shared with: {missing_users}')

        # Delete cluster acl entries
        for acl_id in delete_acl_ids:
            acl_svc.unshare_def_entity(acl_id)

    def list_share_entries(self, cluster_id, cluster_name, org=None, vdc=None):
        if not cluster_id:
            cluster_id = self.get_cluster_id_by_name(cluster_name, org, vdc)

        org_user_id_to_name_dict = vcd_utils.create_org_user_id_to_name_dict(
            self._client, org)
        # Consider system users if client is from system org
        if self._client.is_sysadmin():
            sys_org_user_id_to_name_dict = vcd_utils.create_org_user_id_to_name_dict(  # noqa:E501
                self._client, shared_constants.SYSTEM_ORG_NAME)
            org_user_id_to_name_dict.update(sys_org_user_id_to_name_dict)
        acl_svc = cluster_acl_svc.ClusterACLService(cluster_id, self._client)
        page_num = result_count = 0
        while True:
            page_num += 1
            response_body = acl_svc.get_def_entity_acl_response(
                page_num, cli_constants.CLI_ENTRIES_PER_PAGE)
            result_total = response_body[shared_constants.PaginationKey.RESULT_TOTAL]  # noqa: E501
            values = response_body[shared_constants.PaginationKey.VALUES]
            if len(values) == 0:
                break
            acl_values = []
            for entry in values:
                acl_entry = common_models.ClusterAclEntry(**entry)
                # If there is no username found, the user must be a system
                # user, so a generic name is shown
                acl_entry.username = org_user_id_to_name_dict.get(acl_entry.memberId, shared_constants.SYSTEM_USER_GENERIC_NAME)  # noqa: E501
                acl_values.append(acl_entry.construct_filtered_dict(
                    include=CLUSTER_ACL_LIST_FIELDS))
            result_count += len(values)
            yield acl_values, result_count < result_total



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import container_service_extension.client.cse_client.api_33.native_cluster_api as native_cluster_api_v33  # noqa: E501
import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
from container_service_extension.logging.logger import CLIENT_LOGGER


class LegacyClusterNative:
    def __init__(self, client):
        self._native_cluster_api = \
            native_cluster_api_v33.NativeClusterApi(client)

    def list_clusters(self, vdc=None, org=None):
        filters = {
            shared_constants.RequestKey.ORG_NAME: org,
            shared_constants.RequestKey.OVDC_NAME: vdc}
        for clusters_rep, has_more_results in \
                self._native_cluster_api.get_all_clusters(filters=filters):
            clusters = []
            CLIENT_LOGGER.debug(clusters_rep)
            for c in clusters_rep:
                # TODO cluster api response keys need to be more well defined
                cluster = {
                    'Name': c.get('name', 'N/A'),
                    'Owner': c.get('owner_name', 'N/A'),
                    'VDC': c.get('vdc', 'N/A'),
                    'Org': c.get('org_name', 'N/A'),
                    'K8s Runtime': c.get('k8s_type', 'N/A'),
                    'K8s Version': c.get('k8s_version', 'N/A'),
                    'Status': c.get('status', 'N/A'),
                    'Provider': c.get('k8s_provider', 'N/A'),
                }
                clusters.append(cluster)
            yield clusters, has_more_results

    def get_cluster_info(self, name, org=None, vdc=None, **kwargs):
        filters = {shared_constants.RequestKey.ORG_NAME: org,
                   shared_constants.RequestKey.OVDC_NAME: vdc}
        return self._native_cluster_api.get_cluster(name, filters=filters)

    def get_upgrade_plan(self, cluster_name, org=None, vdc=None):
        filters = {shared_constants.RequestKey.ORG_NAME: org,
                   shared_constants.RequestKey.OVDC_NAME: vdc}
        return self._native_cluster_api.get_cluster_upgrade_plan(cluster_name,
                                                                 filters=filters)  # noqa: E501

    def upgrade_cluster(self, cluster_name, template_name, template_revision,
                        org_name=None, ovdc_name=None):
        return self._native_cluster_api.upgrade_cluster(cluster_name,
                                                        template_name,
                                                        template_revision,
                                                        org_name=org_name,
                                                        ovdc_name=ovdc_name)

    def create_cluster(self,
                       vdc,
                       network_name,
                       name,
                       node_count=None,
                       cpu=None,
                       memory=None,
                       storage_profile=None,
                       ssh_key=None,
                       template_name=None,
                       template_revision=None,
                       enable_nfs=False,
                       rollback=True,
                       org=None):
        """Create a new Kubernetes cluster.

        :param vdc: (str): The name of the vdc in which the cluster will be
            created
        :param network_name: (str): The name of the network to which the
            cluster vApp will connect to
        :param name: (str): The name of the cluster
        :param node_count: (str): The number ofs nodes
        :param cpu: (str): The number of virtual cpus on each of the
            nodes in the cluster
        :param memory: (str): The amount of memory (in MB) on each of the nodes
            in the cluster
        :param storage_profile: (str): The name of the storage profile which
            will back the cluster
        :param ssh_key: (str): The ssh key that clients can use to log into the
            node vms without explicitly providing passwords
        :param template_name: (str): The name of the template to use to
            instantiate the nodes
        :param template_revision: (str): The revision of the template to use to
            instantiate the nodes
        :param enable_nfs: (bool): bool value to indicate if NFS node is to be
            created
        :param rollback: (bool): Flag to control weather rollback
            should be performed or not in case of errors.
        :param org: (str): name of the organization in which the vdc to be
        used for cluster creation.

        :return: (json) A parsed json object describing the requested cluster.
        """
        return self._native_cluster_api.create_cluster(name, vdc, network_name,
                                                       node_count=node_count,
                                                       org_name=org,
                                                       cpu=cpu,
                                                       memory=memory,
                                                       storage_profile=storage_profile,  # noqa: E501
                                                       ssh_key=ssh_key,
                                                       template_name=template_name,  # noqa: E501
                                                       template_revision=template_revision,  # noqa: E501
                                                       enable_nfs=enable_nfs,
                                                       rollback=rollback)

    def resize_cluster(self,
                       network_name,
                       cluster_name,
                       node_count,
                       org=None,
                       vdc=None,
                       rollback=True,
                       template_name=None,
                       template_revision=None,
                       cpu=None,
                       memory=None,
                       ssh_key=None):
        return self._native_cluster_api.update_cluster(cluster_name,
                                                       network_name,
                                                       node_count,
                                                       org_name=org,
                                                       ovdc_name=vdc,
                                                       template_name=template_name,  # noqa: E501
                                                       template_revision=template_revision,  # noqa: E501
                                                       cpu=cpu,
                                                       memory=memory,
                                                       ssh_key=ssh_key,
                                                       rollback=rollback)

    def delete_cluster(self, cluster_name, org=None, vdc=None, **kwargs):
        filters = {
            shared_constants.RequestKey.ORG_NAME: org,
            shared_constants.RequestKey.OVDC_NAME: vdc
        }
        return self._native_cluster_api.delete_cluster(cluster_name,
                                                       filters=filters)

    def get_cluster_config(self, cluster_name, org=None, vdc=None, **kwargs):
        filters = {shared_constants.RequestKey.ORG_NAME: org,
                   shared_constants.RequestKey.OVDC_NAME: vdc}
        return self._native_cluster_api.get_cluster_config(cluster_name,
                                                           filters=filters)

    def get_node_info(self, cluster_name, node_name, org=None, vdc=None):
        filters = {
            shared_constants.RequestKey.ORG_NAME: org,
            shared_constants.RequestKey.OVDC_NAME: vdc,
        }
        return self._native_cluster_api.get_node_info(cluster_name, node_name,
                                                      filters=filters)

    def add_node(self,
                 network_name,
                 cluster_name,
                 node_count=1,
                 org=None,
                 vdc=None,
                 cpu=None,
                 memory=None,
                 storage_profile=None,
                 ssh_key=None,
                 template_name=None,
                 template_revision=None,
                 enable_nfs=False,
                 rollback=True):
        """Add nodes to a Kubernetes cluster.

        :param org: (str): The name of the org that contains the cluster
        :param vdc: (str): The name of the vdc that contains the cluster
        :param network_name: (str): The name of the network to which the
            node VMs will connect to
        :param cluster_name: (str): The name of the cluster
        :param node_count: (str): The number of nodes
        :param cpu: (str): The number of virtual cpus on each of the
            new nodes in the cluster
        :param memory: (str): The amount of memory (in MB) on each of the new
            nodes in the cluster
        :param storage_profile: (str): The name of the storage profile which
            will back the new nodes
        :param ssh_key: (str): The ssh key that clients can use to log into the
            node vms without explicitly providing passwords
        :param template_name: (str): The name of the catalog template to use to
            instantiate the nodes
        :param template_revision: (str): The revision of the template to use to
            instantiate the nodes
        :param enable_nfs: (bool): Flag to enable NFS software on worker nodes
        :param rollback: (bool): Flag to control whether rollback
            should be performed or not in case of errors.

        :return: (json) A parsed json object describing the requested cluster.
        """
        return self._native_cluster_api.add_node(cluster_name, network_name,
                                                 node_count=node_count,
                                                 org_name=org,
                                                 ovdc_name=vdc,
                                                 cpu=cpu,
                                                 memory=memory,
                                                 storage_profile=storage_profile,  # noqa: E501
                                                 ssh_key=ssh_key,
                                                 template_name=template_name,
                                                 template_revision=template_revision)  # noqa: E501

    def delete_nodes(self, cluster_name, nodes, org=None, vdc=None):
        """Delete nodes from a Kubernetes cluster.

        :param str org: Name of the organization that contains the cluster
        :param str vdc: The name of the vdc that contains the cluster
        :param str cluster_name: The name of the cluster
        :param list nodes: The list of nodes to delete

        :return: (json) A parsed json object describing the requested cluster
            operation.
        """
        return self._native_cluster_api.delete_nodes(cluster_name,
                                                     nodes,
                                                     org_name=org,
                                                     ovdc_name=vdc)



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import container_service_extension.common.constants.server_constants as server_constants  # noqa: E501
from container_service_extension.common.constants.server_constants import MQTTExtKey, MQTTExtTokenKey  # noqa: E501
from container_service_extension.common.utils.server_utils import should_use_mqtt_protocol  # noqa: E501
from container_service_extension.mqi.consumer.amqp_consumer import AMQPConsumer
from container_service_extension.mqi.consumer.mqtt_consumer import MQTTConsumer


class MessageConsumer:
    """Returns the consumer class for the correct message protocol."""

    def __new__(cls, config, num_processors):
        """Create the correct message consumer class for the message protocol.

        :param dict config: content of the CSE config file.
        :param int num_processors: number of processors for thread pool
            executor

        :return: instance of appropriate message protocol consumer
        """
        if should_use_mqtt_protocol(config):
            mqtt = config['mqtt']
            return MQTTConsumer(
                url=config['vcd']['host'],
                listen_topic=mqtt[MQTTExtKey.EXT_LISTEN_TOPIC],
                respond_topic=mqtt[MQTTExtKey.EXT_RESPOND_TOPIC],
                verify_ssl=mqtt['verify_ssl'],
                token=mqtt[MQTTExtTokenKey.TOKEN],
                client_username=f'{server_constants.MQTT_EXTENSION_VENDOR}/'
                                f'{server_constants.CSE_SERVICE_NAME}/'
                                f'{server_constants.MQTT_EXTENSION_VERSION}',
                num_processors=num_processors)
        else:
            amqp = config['amqp']
            return AMQPConsumer(
                host=amqp['host'],
                port=amqp['port'],
                vhost=amqp['vhost'],
                username=amqp['username'],
                password=amqp['password'],
                exchange=amqp['exchange'],
                routing_key=amqp['routing_key'],
                num_processors=num_processors)



# container-service-extension
# Copyright (c) 2021 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from container_service_extension.lib.cloudapi.cloudapi_client import CloudApiClient  # noqa: E501
from container_service_extension.rde.behaviors.behavior_model import BehaviorOperation  # noqa: E501
from container_service_extension.rde.validators.abstract_validator import AbstractValidator  # noqa: E501


class Validator_1_0_0(AbstractValidator):
    def __init__(self):
        pass

    def validate(self, cloudapi_client: CloudApiClient, entity_id: str = None,
                 entity: dict = None,
                 operation: BehaviorOperation = None) -> bool:
        """Validate the input entity.

        This method performs
        1. Basic validation of the entity by simply casting the input entity
        dict to the model class dictated by the api_version specified in the
        request.
        2. Operation (create, update, delete) specific validation.

        :param cloudapi_client: cloud api client
        :param dict entity: dict form of the native entity to be validated
        :param entity_id: entity id to be validated
        :param BehaviorOperation operation: CSE operation key
        :return: is validation successful or failure
        :rtype: bool
        """
        raise NotImplementedError



# container-service-extension
# Copyright (c) 2021 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

from dataclasses import dataclass
from typing import List, Optional

from dataclasses_json import dataclass_json
import yaml

import container_service_extension.common.constants.server_constants as server_constants  # noqa: E501
import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
from container_service_extension.rde.models.abstractNativeEntity import AbstractNativeEntity  # noqa: E501
import container_service_extension.rde.models.rde_2_0_0 as rde_2_0_0


@dataclass_json
@dataclass
class Metadata:
    cluster_name: str
    org_name: str
    ovdc_name: str


@dataclass_json
@dataclass
class Nfs:
    sizing_class: Optional[str] = None
    storage_profile: Optional[str] = None
    count: int = 0


@dataclass_json
@dataclass
class ControlPlane:
    sizing_class: Optional[str] = None
    storage_profile: Optional[str] = None
    count: int = 1


@dataclass_json
@dataclass
class Workers:
    sizing_class: Optional[str] = None
    storage_profile: Optional[str] = None
    count: int = 1


@dataclass_json
@dataclass
class Distribution:
    template_name: Optional[str] = ""
    template_revision: int = 0


@dataclass_json
@dataclass
class Settings:
    network: str
    ssh_key: Optional[str] = None
    rollback_on_failure: bool = True


@dataclass_json
@dataclass
class Node:
    name: str
    ip: str
    sizing_class: Optional[str] = None


@dataclass_json
@dataclass
class NfsNode(Node):
    exports: Optional[str] = None


@dataclass_json
@dataclass
class Nodes:
    control_plane: Optional[Node] = None
    workers: Optional[List[Node]] = None
    nfs: Optional[List[NfsNode]] = None


@dataclass_json
@dataclass
class Status:
    # TODO(DEF) Remove master_ip once nodes is implemented.
    phase: Optional[str] = None
    cni: Optional[str] = None
    task_href: Optional[str] = None
    kubernetes: Optional[str] = None
    docker_version: Optional[str] = None
    os: Optional[str] = None
    nodes: Optional[Nodes] = None
    exposed: bool = False


@dataclass_json
@dataclass
class ClusterSpec:
    """Represents the cluster spec.

    If dictionaries are passed as arguments, the constructor auto-converts
    them into the expected class instances.
    """

    settings: Settings
    control_plane: ControlPlane = ControlPlane()
    workers: Workers = Workers()
    nfs: Nfs = Nfs()
    k8_distribution: Distribution = Distribution()
    expose: bool = False


@dataclass_json
@dataclass
class NativeEntity(AbstractNativeEntity):
    """Represents the Native Cluster entity.

    If dictionaries are passed as arguments, the constructor auto-converts
    them into the expected class instances.

    Sample representation in JSON format
    {
        "kind": "native",
        "spec": {
            "workers": {
                "count": 2,
                "sizing_class": "small",
                "storage_profile": "Any"
            },
            "control_plane": {
                "count": 1,
                "sizing_class": "Large",
                "storage_profile": "Any"
            },
            "nfs": {
                "count": 1,
                "sizing_class": "small",
                "storage_profile": "Any"
            },
            "settings": {
                "network": "net",
                "ssh_key": null,
                "rollback_on_failure": true
            },
            "k8_distribution": {
                "template_name": "k81.17",
                "template_revision": 1
            },
            "expose": False
        },
        "status": {
            "cni": null,
            "docker_version": null,
            "exposed": False,
            "kubernetes": null,
            "nodes": {
                "workers": [
                    {
                        "name": "node-abcd",
                        "ip": "10.0.0.2",
                        "sizing_class": "small",
                    }
                ],
                "control_plane": [
                    {
                        "name": "mstr-xyzv",
                        "ip": "10.0.0.3",
                        "sizing_class": "Large",
                    }
                ],
                "nfs": [
                    {
                        "name": "nfsd-uvwx",
                        "ip": "10.0.0.4",
                        "sizing_class": "small",
                        "exports": "/usr/share/abc, /usr/share/xyz"
                    }
                ]
            },
            "os": null,
            "phase": null,
            "task_href" : null
        },
        "metadata": {
            "org_name": "org1",
            "ovdc_name": "ovdc1",
            "cluster_name": "myCluster"
        },
        "api_version": ""
    }
    """

    metadata: Metadata
    spec: ClusterSpec
    status: Status = Status()
    kind: str = shared_constants.ClusterEntityKind.NATIVE.value
    api_version: str = ''

    @classmethod
    def from_native_entity(cls, native_entity: AbstractNativeEntity):
        """Construct rde_1.0.0 native entity.

        :param AbstractNativeEntity native_entity: input native entity
        :return: native entity
        :rtype: rde_1.0.0.NativeEntity
        """
        if isinstance(native_entity, NativeEntity):
            return native_entity

        if isinstance(native_entity, rde_2_0_0.NativeEntity):
            rde_2_x_entity: rde_2_0_0.NativeEntity = native_entity

            metadata = Metadata(
                cluster_name=rde_2_x_entity.metadata.name,
                org_name=rde_2_x_entity.metadata.org_name,
                ovdc_name=rde_2_x_entity.metadata.virtual_data_center_name
            )

            settings = Settings(
                network=rde_2_x_entity.spec.settings.ovdc_network,
                ssh_key=rde_2_x_entity.spec.settings.ssh_key,
                rollback_on_failure=rde_2_x_entity.spec.settings.rollback_on_failure  # noqa: E501
            )

            k8_distribution = Distribution(
                template_name=rde_2_x_entity.spec.distribution.template_name,
                template_revision=rde_2_x_entity.spec.distribution.template_revision  # noqa: E501
            )

            control_plane = ControlPlane(
                sizing_class=rde_2_x_entity.spec.topology.control_plane.sizing_class,  # noqa: E501
                storage_profile=rde_2_x_entity.spec.topology.control_plane.storage_profile,  # noqa: E501
                count=rde_2_x_entity.spec.topology.control_plane.count
            )

            workers = Workers(
                sizing_class=rde_2_x_entity.spec.topology.workers.sizing_class,
                storage_profile=rde_2_x_entity.spec.topology.workers.storage_profile,  # noqa: E501
                count=rde_2_x_entity.spec.topology.workers.count
            )

            nfs = Nfs(
                sizing_class=rde_2_x_entity.spec.topology.nfs.sizing_class,
                storage_profile=rde_2_x_entity.spec.topology.nfs.storage_profile,  # noqa: E501
                count=rde_2_x_entity.spec.topology.nfs.count
            )

            spec = ClusterSpec(
                settings=settings,
                k8_distribution=k8_distribution,
                control_plane=control_plane,
                workers=workers,
                nfs=nfs,
                expose=rde_2_x_entity.spec.settings.network.expose
            )

            # Convert "status" only for entities that are already created
            # New cluster creation won't have "status" section in RDE
            status = cls.status
            if rde_2_x_entity.status.nodes:

                if rde_2_x_entity.status.cloud_properties.exposed:
                    control_plane_ip = rde_2_x_entity.status.external_ip
                else:
                    control_plane_ip = rde_2_x_entity.status.nodes.control_plane.ip  # noqa: E501

                control_plane = Node(
                    name=rde_2_x_entity.status.nodes.control_plane.name,
                    ip=control_plane_ip,
                    sizing_class=rde_2_x_entity.status.nodes.control_plane.sizing_class  # noqa: E501
                )

                workers = []
                for worker_node in rde_2_x_entity.status.nodes.workers:
                    worker_node_1_x = Node(
                        name=worker_node.name,
                        ip=worker_node.ip,
                        sizing_class=worker_node.sizing_class
                    )
                    workers.append(worker_node_1_x)

                nfs_nodes = []
                for nfs_node in rde_2_x_entity.status.nodes.nfs:
                    nfs_node_1_x = NfsNode(
                        name=nfs_node.name,
                        ip=nfs_node.ip,
                        sizing_class=nfs_node.sizing_class,
                        exports=str(nfs_node.exports)
                    )
                    nfs_nodes.append(nfs_node_1_x)

                nodes = Nodes(
                    control_plane=control_plane,
                    workers=workers,
                    nfs=nfs_nodes
                )

                status = Status(
                    phase=rde_2_x_entity.status.phase,
                    cni=rde_2_x_entity.status.cni,
                    task_href=rde_2_x_entity.status.task_href,
                    kubernetes=rde_2_x_entity.status.kubernetes,
                    docker_version=rde_2_x_entity.status.docker_version,
                    os=rde_2_x_entity.status.os,
                    nodes=nodes,
                    exposed=rde_2_x_entity.status.cloud_properties.exposed
                )

            rde_1_entity = cls(
                metadata=metadata,
                spec=spec,
                status=status,
                kind=rde_2_x_entity.kind,
                api_version=''
            )
            return rde_1_entity

        return native_entity

    @classmethod
    def from_cluster_data(cls, cluster: dict, kind: str, **kwargs):
        """Construct rde_1.0.0 native entity from non-rde cluster.

        :param dict cluster: cluster metadata
        :param str kind: cluster kind
        :return: native entity
        :rtype: rde_1.0.0.NativeEntity
        """
        worker_nodes = []
        for item in cluster['nodes']:
            worker_nodes.append(
                Node(name=item['name'], ip=item['ipAddress'])
            )
        nfs_nodes = []
        for item in cluster['nfs_nodes']:
            nfs_nodes.append(
                NfsNode(
                    name=item['name'],
                    ip=item['ipAddress'],
                    exports=item['exports']
                )
            )

        k8_distribution = Distribution(
            template_name=cluster['template_name'],
            template_revision=int(cluster['template_revision'])
        )

        cluster_entity = cls(
            kind=kind,
            spec=ClusterSpec(
                workers=Workers(
                    count=len(cluster['nodes']),
                    storage_profile=cluster['storage_profile_name']
                ),
                control_plane=ControlPlane(
                    count=len(cluster['master_nodes']),
                    storage_profile=cluster['storage_profile_name']
                ),
                nfs=Nfs(
                    count=len(cluster['nfs_nodes']),
                    storage_profile=cluster['storage_profile_name']
                ),
                settings=Settings(
                    network=cluster['network_name'],
                    ssh_key=""
                ),
                k8_distribution=k8_distribution
            ),
            status=Status(
                phase=str(server_constants.DefEntityPhase(
                    server_constants.DefEntityOperation.CREATE,
                    server_constants.DefEntityOperationStatus.SUCCEEDED)
                ),
                kubernetes=f"{cluster['kubernetes']} {cluster['kubernetes_version']}",  # noqa: E501
                cni=f"{cluster['cni']} {cluster['cni_version']}",
                os=cluster['os'],
                docker_version=cluster['docker_version'],
                nodes=Nodes(
                    control_plane=Node(
                        name=cluster['master_nodes'][0]['name'],
                        ip=cluster['master_nodes'][0]['ipAddress']
                    ),
                    workers=worker_nodes,
                    nfs=nfs_nodes
                )
            ),
            metadata=Metadata(
                org_name=cluster['org_name'],
                ovdc_name=cluster['vdc_name'],
                cluster_name=cluster['name']
            ),
            api_version=""
        )
        return cluster_entity

    @classmethod
    def get_sample_native_cluster_specification(cls, k8_runtime: str = shared_constants.ClusterEntityKind.NATIVE.value):  # noqa: E501
        cluster_spec_field_descriptions = """# Short description of various properties used in this sample cluster configuration
# api_version: Represents the payload version of the cluster specification. By default, empty.
# kind: The kind of the Kubernetes cluster.
#
# metadata: This is a required section
# metadata.cluster_name: Name of the cluster to be created or resized.
# metadata.org_name: The name of the Organization in which cluster needs to be created or managed.
# metadata.ovdc_name: The name of the Organization Virtual data center in which the cluster need to be created or managed.
#
# spec: User specification of the desired state of the cluster.
# spec.control_plane: An optional sub-section for desired control-plane state of the cluster. The properties \"sizing_class\" and \"storage_profile\" can be specified only during the cluster creation phase. These properties will no longer be modifiable in further update operations like \"resize\" and \"upgrade\".
# spec.control_plane.count: Number of control plane node(s). Only single control plane node is supported.
# spec.control_plane.sizing_class: The compute sizing policy with which control-plane node needs to be provisioned in a given \"ovdc\". The specified sizing policy is expected to be pre-published to the given ovdc.
# spec.control_plane.storage_profile: The storage-profile with which control-plane needs to be provisioned in a given \"ovdc\". The specified storage-profile is expected to be available on the given ovdc.
#
# spec.k8_distribution: This is a required sub-section.
# spec.k8_distribution.template_name: Template name based on guest OS, Kubernetes version, and the Weave software version
# spec.k8_distribution.template_revision: revision number
#
# spec.nfs: Optional sub-section for desired nfs state of the cluster. The properties \"sizing_class\" and \"storage_profile\" can be specified only during the cluster creation phase. These properties will no longer be modifiable in further update operations like \"resize\" and \"upgrade\".
# spec.nfs.count: Nfs nodes can only be scaled-up; they cannot be scaled-down. Default value is 0.
# spec.nfs.sizingClass: The compute sizing policy with which nfs node needs to be provisioned in a given \"ovdc\". The specified sizing policy is expected to be pre-published to the given ovdc.
# spec.nfs.storageProfile: The storage-profile with which nfs needs to be provisioned in a given \"ovdc\". The specified storage-profile is expected to be available on the given ovdc.
#
# spec.settings: This is a required sub-section
# spec.settings.network: This value is mandatory. Name of the Organization's virtual data center network
# spec.settings.rollback_on_failure: Optional value that is true by default. On any cluster operation failure, if the value is set to true, affected node VMs will be automatically deleted.
# spec.settings.ssh_key: Optional ssh key that users can use to log into the node VMs without explicitly providing passwords.
#
# spec.workers: Optional sub-section for the desired worker state of the cluster. The properties \"sizing_class\" and \"storage_profile\" can be specified only during the cluster creation phase. These properties will no longer be modifiable in further update operations like \"resize\" and \"upgrade\". Non uniform worker nodes in the clusters is not yet supported.
# spec.workers.count: number of worker nodes (default value:1) Worker nodes can be scaled up and down.
# spec.workers.sizing_class: The compute sizing policy with which worker nodes need to be provisioned in a given \"ovdc\". The specified sizing policy is expected to be pre-published to the given ovdc.
# spec.workers.storage_profile: The storage-profile with which worker nodes need to be provisioned in a given \"ovdc\". The specified storage-profile is expected to be available on the given ovdc.
#
# status: Current state of the cluster in the server. This is not a required section for any of the operations.\n
"""  # noqa: E501
        metadata = Metadata('cluster_name', 'organization_name',
                            'org_virtual_data_center_name')
        status = Status()
        settings = Settings(network='ovdc_network_name', ssh_key=None)
        k8_distribution = Distribution(
            template_name='ubuntu-16.04_k8-1.17_weave-2.6.0',
            template_revision=2
        )
        control_plane = ControlPlane(
            count=1,
            sizing_class='Large_sizing_policy_name',
            storage_profile='Gold_storage_profile_name'
        )
        workers = Workers(
            count=2,
            sizing_class='Medium_sizing_policy_name',
            storage_profile='Silver_storage_profile'
        )

        nfs = Nfs(
            count=0,
            sizing_class='Large_sizing_policy_name',
            storage_profile='Platinum_storage_profile_name'
        )

        cluster_spec = ClusterSpec(
            control_plane=control_plane,
            k8_distribution=k8_distribution,
            settings=settings,
            workers=workers,
            nfs=nfs
        )

        native_entity_dict = NativeEntity(metadata=metadata,
                                          spec=cluster_spec,
                                          status=status,
                                          kind=k8_runtime).to_dict()
        # remove status part of the entity dict
        del native_entity_dict['status']

        sample_apply_spec = yaml.dump(native_entity_dict)
        return cluster_spec_field_descriptions + sample_apply_spec
